@@ -1,34 +1,28 @@
-from investing_algorithm_framework.utils.version import get_version
+from investing_algorithm_framework.app import App
+from investing_algorithm_framework.configuration import Config
 from investing_algorithm_framework.core.context import \
     AlgorithmContextInitializer, AlgorithmContext
 from investing_algorithm_framework.core.data_providers import DataProvider
-from investing_algorithm_framework.core.portfolio_managers import \
-    PortfolioManager, BinancePortfolioManager, SQLLitePortfolioManager
-from investing_algorithm_framework.core.order_executors import OrderExecutor, \
-    BinanceOrderExecutor
-from investing_algorithm_framework.core.market_services import \
-    MarketService, BinanceMarketService
-from investing_algorithm_framework.core.models.data_provider import \
-    TradingDataTypes, OrderBook, Ticker
+from investing_algorithm_framework.core.exceptions import \
+    OperationalException, ImproperlyConfigured
+from investing_algorithm_framework.core.market_services import MarketService
 from investing_algorithm_framework.core.models import OrderSide, Order, \
     Position, TimeUnit, db, Portfolio, OrderType, OrderStatus, \
     SQLLitePortfolio, SQLLiteOrder, SQLLitePosition
-from investing_algorithm_framework.core.exceptions import \
-    OperationalException, ImproperlyConfigured
-from investing_algorithm_framework.app import App
-from investing_algorithm_framework.globals import current_app
-from investing_algorithm_framework.configuration import Config
+from investing_algorithm_framework.core.models.data_provider import \
+    TradingDataTypes, OrderBook, Ticker
+from investing_algorithm_framework.core.order_executors import OrderExecutor
+from investing_algorithm_framework.core.portfolio_managers import \
+    PortfolioManager, SQLLitePortfolioManager
 from investing_algorithm_framework.exceptions import ApiException
+from investing_algorithm_framework.globals import current_app
+from investing_algorithm_framework.utils.version import get_version
 from investing_algorithm_framework.views import *
 
-VERSION = (0, 16, 8, 'alpha', 0)
- 
- 
 __all__ = [
     "App",
     'get_version',
     'PortfolioManager',
-    'BinancePortfolioManager',
     "AlgorithmContextInitializer",
     "Portfolio",
     "OrderSide",
@@ -40,20 +34,18 @@ __all__ = [
     "current_app",
     "AlgorithmContext",
     "OrderExecutor",
-    'BinanceOrderExecutor',
     "Config",
     "MarketService",
-    "BinanceMarketService",
     "OrderStatus",
     "TradingDataTypes",
     "Ticker",
     "OrderBook",
     "OperationalException",
     "ImproperlyConfigured",
-    "SQLLitePortfolioManager",
     "DataProvider",
     "SQLLitePortfolio",
     "SQLLiteOrder",
     "SQLLitePosition",
-    "ApiException"
+    "ApiException",
+    "SQLLitePortfolioManager"
 ]
